"""
Curses PLus is an extension to the curses module that provides some useful featues. This library will be distributed as part of many Enderbyte Programs software
(c) 2022-2023 Enderbyte Programs, no rights reserved
"""

__version__ = "1.3.1"
__author__ = "Enderbyte Programs"
__package__ = "cursesplus"
from .cp import *